# -*- coding: utf8 -*-

from __future__ import unicode_literals

import importlib
import os
import sys

from driverng.message import Message, MessageV2, GraphContainer, JSONGraphContainer
from driverng.worklist import Worklist
from driverng.config import Config, ConfigError

__all__ = ["Message", "MessageV2", "GraphContainer", "JSONGraphContainer", "Worklist", "Config", "ConfigError"]

_DRIVERS = {}
_DRIVERS_LOADED = False


if not getattr(sys, "frozen", False):

    def load_drivers():
        global _DRIVERS
        for mod_path in os.listdir(os.path.join(
                os.path.dirname(__file__),
                "drivers")):
            abs_mod_path = os.path.join(
                os.path.dirname(__file__),
                "drivers",
                mod_path,
            )
            if os.path.isdir(abs_mod_path) and mod_path != "__pycache__":
                modname = "driverng.drivers.%s" % mod_path
            elif mod_path.endswith("_spec.py"):
                modname = "driverng.drivers.%s" % mod_path.rsplit(".")[0]
            else:
                modname = None
            if modname is not None:
                mod = importlib.import_module(modname)
                if hasattr(mod, "Driver"):
                    if isinstance(mod.Driver, list):
                        drivers = mod.Driver
                    else:
                        drivers = [mod.Driver]
                    for klass in drivers:
                        _DRIVERS[klass.SPEC["code"]] = klass

    def load_required(f):
        def wrapper(*a, **kw):
            global _DRIVERS_LOADED
            if not _DRIVERS_LOADED:
                load_drivers()
                _DRIVERS_LOADED = True
            return f(*a, **kw)
        return wrapper

    @load_required
    def get(code):
        global _DRIVERS
        return _DRIVERS[code]

    @load_required
    def get_labels():
        global _DRIVERS
        return sorted([
            (k.SPEC['code'], k.SPEC['label'])
            for k in _DRIVERS.values()], key=lambda x: x[1])

    @load_required
    def get_formats():
        global _DRIVERS
        return sorted([{
            "code": v.SPEC["code"],
            "label": v.SPEC["label"],
            "options": v.get_options(),
        } for v in _DRIVERS.values()], key=lambda x: x["label"])
