# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.drivers.celldyn.ruby import (
    DriverRuby,
    RubySegmentR,
    RubySegmentO,
    RubyGraphContainer,
)
from driverng.astm import (
    Segment,
    HprimString,
)


class SapphireGraphContainter(RubyGraphContainer):
    SCATTOGRAM_COLOR_MAP = {
        "lymph": "y,",
        "mono": "c,",
        "baso": "b,",
        "neutro": "g,",
        "eos": "w,",
        "rbc": "m,",
    }


class SapphireSegmentO(RubySegmentO):

    rack = HprimString((3, 2))
    position = HprimString((3, 3))


class SapphireSegmentR(RubySegmentR):
    test = HprimString((3, 8))


class SapphireSegmentC(Segment):
    # comment source
    code_commentaire = HprimString(3)

    comment_type = HprimString((4, 1))
    comment_code = HprimString((4, 2))
    comment_flag = HprimString((4, 3))
    comment_value = HprimString((4, 4))

    # type de commentaire
    type_commentaire = HprimString(5)


class DriverSapphire(DriverRuby):
    SPEC = {
        "code": "sapphire",
        "label": "Celldyn Sapphire",
    }
    SEGMENT_CLASS_MAP = dict(DriverRuby.SEGMENT_CLASS_MAP.items() + {
        "O": SapphireSegmentO,
        "R": SapphireSegmentR,
        "C": SapphireSegmentC,
    }.items())
    GRAPH_CLASS = SapphireGraphContainter

    @staticmethod
    def get_rack(seg_o):
        if seg_o.rack and seg_o.position:
            return "%s-%s" % (
                seg_o.rack,
                seg_o.position,
            )
        return None

    @staticmethod
    def ignore_resultat(seg_r):
        if seg_r.status != "F":
            return True
        else:
            return False

    def get_alarmes_hemato(self, message):
        alarmes_hemato = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                if "QC" in seg_o.type_echantillon:
                    continue

                analyseur = None
                for seg_r in seg_o.results:
                    if seg_r.analyseur and analyseur is None:
                        analyseur = seg_r.analyseur.strip()
                    elif seg_r.analyseur and seg_r.analyseur != analyseur:
                        raise RuntimeError((
                            "Impossible de déduire l'analyseur pour les"
                            "alarmes hemato, analyseurs multiples !!"))
                for seg_c in seg_o.commentaires:
                    if (
                        (seg_c.comment_type, seg_c.comment_flag) == ("SP", "YES") and
                        seg_c.comment_code
                    ):
                        alarmes_hemato.setdefault(seg_o.tube, {
                            "alarmes": [],
                            "analyseur": analyseur,
                        })["alarmes"].append(seg_c.comment_code)
        return alarmes_hemato
