# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import csv
from StringIO import StringIO

from driverng import MessageV2
from driverng import Worklist
from driverng.splitted import Splitted, String


class DrDotBase(Splitted):
    SEPARATOR = ";"


class DrDotResult(DrDotBase):
    FIELDS = [
        String("feuille_resultat"),
        String("operateur"),
        String("date"),
        String("kit"),
        String("lot"),
        String("nom"),
        String("antigene1"),
        String("valeur1"),
        String("antigene2"),
        String("valeur2"),
        String("antigene3"),
        String("valeur3"),
        String("antigene4"),
        String("valeur4"),
        String("antigene5"),
        String("valeur5"),
        String("antigene6"),
        String("valeur6"),
        String("antigene7"),
        String("valeur7"),
        String("antigene8"),
        String("valeur8"),
        String("antigene9"),
        String("valeur9"),
        String("antigene10"),
        String("valeur10"),
        String("antigene11"),
        String("valeur11"),
        String("antigene12"),
        String("valeur12"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            setattr(self, field.name, line[i])


class DrDotMultiple(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: DrDotResult,
    }

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data.replace(u'\x99', '').replace(u'\xb0', '').replace(u'\xe9', '').replace(u'\xe8', ''))
        reader = csv.reader(buf, delimiter=str(u';'), quotechar=str(u'"'))
        for line in reader:
            if "Feuille" in line[0]:
                continue
            if not len(line):
                continue
            if len(line) == 30:
                type_message = MessageV2.RESULT
            obj = self.add(type_message=type_message)
            obj.parse(line)


class Driver(MessageV2):
    SPEC = {
        "code": "dr_dot",
        "label": "Dr Dot-Blue Diver",
    }

    def create(self):
        return DrDotMultiple()

    def parse(self, data):
        obj = DrDotMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            sample = worklist.get_or_create(
                sample_id=m.nom,
                type="patient",
            )
            i = 1
            while i <= 12:
                # Traitement des résultats des 12 antigènes
                antigene = getattr(m, "antigene{}".format(i))
                sample.results.add(
                    code="{}{}".format(m.kit, antigene),
                    value=getattr(m, "valeur{}".format(i)),
                    date=datetime.datetime.strptime(m.date, "%d/%m/%y %H:%M"),
                    operator=m.operateur
                )
                i += 1
        return worklist
