# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import xml.etree.cElementTree as ET

from driverng import Message
from driverng import utils


class Driver(Message):
    SPEC = {
        "code": "filmarray",
        "label": "Filmarray Biomerieux",
    }
    ENCODING = "utf8"

    def parse(self, data):
        return ET.fromstring(data.encode(self.ENCODING))

    def to_string(self, message):
        return utils.prettify_xml(ET.tostring(message, 'utf-8'))

    def get_message_type(self, message):
        return self.ORDER

    def get_result_dict(self, message):
        results = {}
        for request in message.findall("requestResult"):
            for test_order in request.findall("testOrder"):
                sample_id = test_order.find(
                    "specimen").find("specimenIdentifier").text
                for test in test_order.findall("test"):
                    resultats = []
                    for result in test.findall("resultGroup"):
                        for res in result.findall("result"):
                            # code test du résultat
                            resultid = res.find("resultID")
                            testid = resultid.find("resultTestCode").text

                            # résultat
                            value = res.find("value")
                            testresult = value.find("testResult")
                            resu = testresult.find("observationValue").text
                            date = datetime.datetime.strptime(
                                res.find("resultDateTime").text,
                                "%Y%m%d%H%M%S",
                            )
                            resultats.append((testid, resu, date))

                    lots_reactifs = []
                    for disposableData in test.findall("disposableData"):
                        for disposable in disposableData.findall("disposable"):
                            lot = disposable.findtext("lotNumber")
                            if lot:
                                lot = {"ref": lot}
                                lots_reactifs.append(lot)
                    for code, resu, date in resultats:
                        results.setdefault(
                            sample_id, {"resultats": []},
                        )["resultats"].append({
                            "test": code,
                            "resu": resu,
                            "date": date,
                            "lots_reactifs": lots_reactifs,
                        })
        return results
