# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.drivers.gem4000 import DriverGem4000
from driverng import Worklist


class DriverGemWeb(DriverGem4000):
    SPEC = {
        "code": "gemweb",
        "label": "GemWeb Werfen"
    }
    DEFAULT_SEPARATORS = "|@^\\"

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            message_id=context["now"],
            date=context["now"]
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={'del': worklist.IGNORE, worklist.ELSE: ""},
        ):
            for sample, action in patient.samples:
                msg.patients.add(
                    patient_lab_id=patient.lab_patient_id,
                    last_name=patient.last_name,
                    first_name=patient.first_name,
                    birthdate=patient.birthdate,
                    sex=patient.sex,
                    special_field1=sample.sample_id,
                    special_field2="SAMPLE_NUMBER",
                )
        if not msg.patients:
            msg.queries = context["query"].queries
            msg.queries[0].request_info = 'X'
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                if seg_q.specimen_id == 'SAMPLE_NUMBER':
                    worklist.get_or_create(
                        type="patient",
                        sample_id=seg_q.sample_no
                    )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:

                for seg_o in seg_p.orders:
                    if seg_o.sample.nlot_descriptor[:7] == "GEM CVP":
                        sample = worklist.get_or_create(
                            type="control",
                            control_lot=seg_o.sample.nlot,
                            sample_id=seg_o.sample.nlot,
                            control_level=seg_o.sample.lot_level,
                        )
                    else:
                        sample = worklist.get_or_create(
                            type="patient",
                            sample_id=seg_o.sample_no,
                        )

                    date = None
                    operator = None

                    for seg_r in seg_o.results:
                        test = seg_r.universal_test_id

                        if seg_r.operator_identification:
                            operator = seg_r.operator_identification

                        if seg_r.date_test_completed:
                            date = seg_r.date_test_completed

                        flags = set(seg_r.abnormal_flags[:])
                        for seg_c in seg_r.commentaires:
                            if seg_c.comment_code:
                                flags.add(seg_c.comment_code)

                        result = sample.results.add(
                            code=test,
                            value=seg_r.result,
                            flags=sorted(flags),
                            date=date,
                            instrument=seg_r.analyser or message.sender.analyser_name,
                        )
                        if sample.type == 'patient' and operator:
                            result.operator = operator
                        if seg_r.cartridge_number:
                            result.lots.add(
                                type="reagent",
                                lot=seg_r.cartridge_number,
                            )

                    if sample.type == "patient" and sample.results:

                        comments = []
                        for seg_c in seg_o.commentaires:
                            if seg_c.comment_code == 'COMMENT' and seg_c.comment_text:
                                comments.append(seg_c.comment_text)
                        if comments:
                            sample.results.add(
                                code='COMMENT',
                                value=";".join(comments),
                                date=date,
                                instrument=message.sender.analyser_name,
                            )
                        if operator:
                            sample.results.add(
                                code='OPE',
                                value=operator,
                                date=date,
                                instrument=message.sender.analyser_name,
                            )
                        if seg_o.sample.sample_type:
                            sample.results.add(
                                code='PRELEV',
                                value=seg_o.sample.sample_type,
                                date=date,
                                instrument=message.sender.analyser_name,
                            )
        return worklist
