# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.drivers.genexpert import(
    DriverGenexpert,
)


class DriverGenexpertV46(DriverGenexpert):
    SPEC = {
        "code": "genexpert_v46",
        "label": "Genexpert Version Protocole 4.6"
    }
    DEFAULT_SEPARATORS = "|@^\\"

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(
            message_id=context["now"],
            date=context["now"]
        )
        msg.set_raw("sender_name", self.code_emetteur)
        msg.set_raw("receiver", self.code_recepteur)
        msg.end.terminaison_code = "F"
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "A"},
        ):
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                patient_last_name=patient.last_name,
                patient_first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex or "U",
            )
            for sample, action, priority in patient.samples:
                for order in sample.orders:
                    seg_o = seg_p.orders.add(
                        specimen_id=sample.sample_id,
                        test=order.code,
                        priority=priority,
                        action_code=action,
                        ordered_date=sample.request_date,
                        specimen_descriptor=sample.specimen,
                        report_type="Q",
                    )
                    seg_o.specimen_descriptor = "ORH"
        if msg.patients:
            return msg
        elif context["query"] and context["query"].queries:
            msg.end.terminaison_code = "I"
            return msg
