# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Message
from driverng.fixed import (
    FixedLengthFormat,
    String,
    Date,
)


class Ha81(FixedLengthFormat):
    FMT_DESCRIPTION = [
        "NO.",
        String("mesure", 4),
        " BC.",
        String("patient_id", 13),
        1,
        Date("date", 15, fmt_date="%y-%m-%d\r\n%H:%M"),
        1,
        "HbA1c",
        String("hba1c", 4),
        1,
        "HbA1",
        String("hba1", 4),
        1,
        "HbF",
        String("hbf", 4),
        1,
        String("erreur", 1),
        "\r\n",
    ]


class Driver(Message):
    SPEC = {
        "code": "ha81",
        "label": "HA81 Uni (HA8160, HA8140, HA8121)",
    }

    def create(self, **kwargs):
        return Ha81(**kwargs)

    def get_message_type(self, message):
        return self.ORDER

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = Ha81()
        obj.parse(data)
        return obj

    def get_result_dict(self, message):
        results = []
        if message.erreur:
            alarmes = [message.erreur]
        else:
            alarmes = []
        for code, attr in (
            ("HbA1c", "hba1c"),
            ("HbA1", "hba1"),
            ("HbF", "hbf"),
        ):
            results.append({
                "test": code,
                "resu": getattr(message, attr),
                "date": message.date,
                "alarmes": alarmes,
            })
        tube = message.patient_id.replace("-", "")
        return {tube: {"resultats": results}}
