# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.astm import (
    AstmMessage,
    HeaderSegment,
    Segment,
    HprimString,
    HprimForeign,
)


class HemofaxSegmentH(HeaderSegment):

    patients = HprimForeign("P", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True)


class HemofaxSegmentP(Segment):

    orders = HprimForeign("O", "patient")


class HemofaxSegmentO(Segment):

    tube = HprimString(3)
    results = HprimForeign("R", "order")


class HemofaxSegmentR(Segment):

    test = HprimString((3, 4))
    valeur = HprimString(4)
    status = HprimString(9)


class HemofaxSegmentL(Segment):
    pass


class Driver(AstmMessage):
    SPEC = {
        "code": "hemofax",
        "label": "Hemofax",
    }
    SEGMENT_CLASS_MAP = {
        "H": HemofaxSegmentH,
        "P": HemofaxSegmentP,
        "O": HemofaxSegmentO,
        "R": HemofaxSegmentR,
        "L": HemofaxSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = HemofaxSegmentH(type_segment="H", **kwargs)
        message.end = HemofaxSegmentL(type_segment="L")
        return message

    def get_message_type(self, message):
        return self.ORDER

    def get_result_dict(self, message):
        results = {}
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                # HACK sur CAUTO à remplacer
                results.setdefault(seg_o.tube, {
                    "resultats": []})["resultats"].append(
                    {
                        "test": "LAME",
                        "resu": "HFAXS",
                    })
                for seg_r in seg_o.results:

                    if seg_r.status != "F":
                        continue

                    results[seg_o.tube]["resultats"].append({
                        "test": seg_r.test,
                        "resu": seg_r.valeur,
                        "zero": ["0", "0.0", "", "NO"],
                    })

        return results
