# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging
from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    Segment,
    HeaderSegment,
    HprimString,
    HprimMultiple,
    HprimDate,
    HprimPN,
    HprimList,
    HprimAD,
    HprimCNA,
    HprimFloat,
    HprimCE,
    HprimForeign,
)

logger = logging.getLogger("driverng.hprim")


class HprimSegmentH(HeaderSegment):

    nom_fichier = HprimString(3, max_length=12)
    emetteur = HprimMultiple(5,
                             code=HprimString(1, obligatoire=True),
                             nom=HprimString(2, obligatoire=True),
                             )

    adresse = HprimAD(6)

    # ORM: Demande
    # ORU: Resultat
    # ADM: Admission
    # FAC: Facturation
    # REG: Reglement
    # ERR: Erreur
    type_message = HprimString(7, obligatoire=True, max_length=7)
    telephone = HprimList(8)

    recepteur = HprimMultiple(10,
                              code=HprimString(1, obligatoire=True),
                              nom=HprimString(2, obligatoire=True),
                              )

    # P: Production
    # T: Test
    # D: Debug
    mode_traitement = HprimString(12, max_length=1, default="P")
    version = HprimString((13, 1), default="H2.1")
    type_liaison = HprimString((13, 2), default="L")
    date = HprimDate(14, obligatoire=True)

    patients = HprimForeign("P", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True)


class HprimSegmentP(Segment):

    patient_demandeur = HprimString((3, 1), max_length=16)
    patient_fusion = HprimString((3, 2))
    patient_executant = HprimString((4, 1), max_length=16)
    no_adm_cli = HprimString((5, 1))
    rang_naissance = HprimString((5, 2), max_length=2)
    nom = HprimPN(6, max_length=48)
    nomjf = HprimString(7, max_length=24)
    date_nai = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sexe = HprimString(9, max_length=1)
    adresse = HprimAD(11)
    telephone = HprimList(13)
    medecins = HprimList(14, basefield=HprimCNA(0))
    traite_lc1 = HprimString(15)
    traite_lc2 = HprimString(16)
    taille = HprimString(17)
    poids = HprimFloat(18)
    diag_pat = HprimString(19)
    diagnostic_patient = HprimList(19, basefield=HprimCE(0))
    trait_medical = HprimString(20)
    traitement_medical = HprimList(20)
    regime = HprimString(21)
    comment_dem1 = HprimString((22, 1))
    comment_dem1_libelle = HprimString((22, 2))
    comment_dem2 = HprimString(23)
    date_entree = HprimDate((24, 1), fmt_date="%Y%m%d")
    date_sortie = HprimDate((24, 2), fmt_date="%Y%m%d")
    stat_adm = HprimString(25, max_length=2)
    localisation = HprimMultiple(26, max_length=100,
                                 lit=HprimString(1),
                                 chambre=HprimString(2),
                                 corr=HprimString(3),
                                 )
    class_diag = HprimString(27)
    situ_maritale = HprimString(29)
    precaution = HprimString(30)
    langue = HprimString(31)
    stat_conf = HprimString(32)
    date_modif = HprimDate(33, fmt_date="%Y%m%d")
    date_deces = HprimDate(34, fmt_date="%Y%m%d")

    orders = HprimForeign("OBR", "order")


class HprimSegmentOBR(Segment):

    tube_demandeur = HprimString((3, 1))
    no_demandeur = HprimString((3, 2))
    tube_executant = HprimString((4, 1))
    no_executant = HprimString((4, 2))
    prescription = HprimList(
        5, basefield=HprimCE(0), obligatoire=True)

    # S: Très urgent
    # A: Urgent
    # R: Routine
    # C: Rappel
    # P: Pré-opératoire
    priorite = HprimString((6, 1))
    date_demande = HprimDate(7)
    date_prel = HprimDate(8)
    date_fin_prel = HprimDate(9)
    volume = HprimString(10, max_length=10)
    preleveur = HprimCNA(11, max_length=60)

    # A: Ajout
    # C: Annulation
    # N: Nouvelle demande
    # L: Nouvelle demande avec prélèvement à effectuer par l'éxecutant
    # R: Modification de demande
    action = HprimString(12)
    risque = HprimString(13, max_length=60)
    renseignement_clinique = HprimString(14, max_length=300)
    date_recu = HprimDate(15)
    nature = HprimMultiple(16, max_length=300,
                           type_tube=HprimMultiple(1,
                                                   code=HprimString(1),
                                                   label=HprimString(2),
                                                   table=HprimString(3),
                                                   ),
                           additif=HprimString(2),
                           methode_prelevement=HprimString(3),
                           )
    prescripteur = HprimList(17, basefield=HprimMultiple(
        0,
        code_medecin=HprimString(1),
        medecin=HprimPN(2),
        type_medecin=HprimString(3),
        code_service=HprimString(4),
        service=HprimPN(5),
        type_service=HprimString(6),

        # Legacy attributes
        nom_medecin=HprimString(2),
        nom_service=HprimString(5),
    ))
    tel_prescrip = HprimList(18)
    lib1_dem = HprimString(19, max_length=60)
    lib2_dem = HprimString(20, max_length=60)
    lib1_exe = HprimString(21, max_length=60)
    lib2_exe = HprimString(22, max_length=60)
    date_resu = HprimDate(23, fmt_date="%Y%m%d%H%M%S")
    service_executant = HprimMultiple(25,
                                      code=HprimString(1),
                                      label=HprimString(2),
                                      )
    stat_ana = HprimString(26, max_length=1)
    dest_copie = HprimList(29)
    mod_trans_pat = HprimString(31, max_length=20)
    valideur = HprimCNA(33)
    assistant = HprimCNA(34)
    technicien = HprimCNA(35)
    ope_saisie = HprimCNA(36)
    date_prv_rendu = HprimDate(37)

    results = HprimForeign("OBX", "order")

    def get_action(self):
        return {
            "C": "del",
            "A": "add",
        }.get(self.action, "new")

    def get_services(self):
        if self.prescripteur and self.prescripteur[0].code_service:
            prescripteur = self.prescripteur[0]
            return [{
                "code": prescripteur.code_service,
                "name": prescripteur.service.nom,
            }]
        # Si le champ OBR.17.4 est vide, on suppose que le SIL nous met
        # l'information dans le champ OBR.25.1
        elif self.service_executant.code:
            return [{
                "code": self.service_executant.code,
                "name": self.service_executant.label,
            }]
        else:
            return []

    def get_comments(self):
        return [e for e in [
            self.renseignement_clinique,
            self.lib1_dem, self.lib2_dem,
        ] if e]


class HprimSegmentOBX(Segment):

    type_resu = HprimString(3, max_length=3)
    code = HprimString((4, 1), max_length=22)
    libelle = HprimString((4, 2), max_length=60)
    table_reference = HprimString((4, 3))  # L pour codification locale
    code2 = HprimString((4, 4), max_length=22)
    sous_test = HprimString(5, max_length=20)
    resu = HprimList(6, basefield=HprimCE(0))
    unite = HprimCE(7, max_length=20)
    normale = HprimString((8, 1))
    indic_norm = HprimList(9)
    probabilite = HprimString(10, max_length=5)
    nature_norm = HprimString((11, 1))
    stat_resu = HprimString(12, max_length=2)
    date_alt = HprimDate(13)
    droit_acces = HprimString(14, max_length=20)
    date = HprimDate(15)
    secteur_tech = HprimMultiple(
        16,
        libelle=HprimString(1),
        rang=HprimString(2),
        code_batterie=HprimString(3),
        rang_batterie=HprimString(4),
        rang_test=HprimString(5),
        text=HprimString(6),
    )
    valideur = HprimString(17, max_length=60)

    # Si ces deux champs sont renseignés, le graphe est à pas fixe et le champ
    # 10.6 contient uniquement les ordonnées des points
    gc_pas = HprimString((18, 3))
    gc_origine = HprimString((18, 4))

    # HORS NORME !!
    analyseur = HprimString(19)
    resultat_controle = HprimString(20)
    # Information demande antérieure Clarilab
    no_demande = HprimString((21, 1))
    date_demande = HprimDate((21, 2))


class HprimSegmentL(Segment):

    nombre_segment_patient = HprimString(4, max_length=4)
    nombre_segment = HprimString(5, max_length=10)
    numero_lot = HprimString(6, max_length=12)


class HprimSegmentC(Segment):
    # P => Émis par le demandeur
    # L => Remis par l'éxecutant
    origine = HprimString(3, max_length=1)
    texte = HprimList(4)


class Driver(AstmMessageV2):
    SPEC = {
        "code": "hprim",
        "label": "Hprim 2.1",
    }
    DEFAULT_SEPARATORS = "|~^\&"

    SEGMENT_CLASS_MAP = {
        "H": HprimSegmentH,
        "P": HprimSegmentP,
        "OBR": HprimSegmentOBR,
        "OBX": HprimSegmentOBX,
        "L": HprimSegmentL,
        "C": HprimSegmentC,
    }
    ADDENDUM_SEGMENT_SIZE = 220

    OPTIONS = [
        AstmMessageV2.Option("hprim_code_emetteur", label="Code emetteur", default="MCA"),
        AstmMessageV2.Option("hprim_nom_emetteur", label="Nom emetteur"),
        AstmMessageV2.Option("hprim_code_recepteur", label="Code recepteur"),
        AstmMessageV2.Option("hprim_nom_recepteur", label="Nom recepteur"),
        AstmMessageV2.BooleanOption(
            "option_comments_in_flags",
            label="Intégrer les commentaires OBX en alarme",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_envoi_technicien",
            label="Envoyer le code technicien transmis par l'automate au lieu du code du valideur en cas de validation automatique",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_envoi_sample_id",
            label="Multi SIL (OBR.9.3.2=No tube)",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_envoi_commentaire_patient",
            label="Envoyer le commentaire patient avec l'envoi de la demande",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_ipp_executant",
            label="Renvoyer l'IPP du patient dans le champs 8.4",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_envoyer_fic",
            label="Envoyer les pièces jointes de la demande dans des segments OBX",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_identification_demandeur",
            label="Identification demandeur OBR.9.19",
            default=False),
        AstmMessageV2.IntegerOption(
            "option_annulation_sans_ipp_age_max",
            label="Accepter les demandes d'annulation sans IPP ni OBR 9.3.2. Préciser l'age maximum de la demande (en jour).",
            default=0),
        AstmMessageV2.BooleanOption(
            "option_envoi_code_groupe",
            label="Envoyer le code groupe du demandeur dans le segment H.5.1",
            default=False),
        AstmMessageV2.BooleanOption(
            "option_bioserveur",
            label="Active les spécificités bioserveur",
            default=False)
    ]

    def __init__(
        self, hprim_code_emetteur="MCA", hprim_code_recepteur="",
        hprim_nom_emetteur="", hprim_nom_recepteur="",
        option_comments_in_flags=False, option_envoi_technicien=False,
        option_envoi_sample_id=False, option_ipp_executant=False,
        option_envoyer_fic=False, option_identification_demandeur=False,
        option_annulation_sans_ipp_age_max=0, option_envoi_commentaire_patient=False,
        option_envoi_code_groupe=False, **kwargs
    ):
        self.code_emetteur = hprim_code_emetteur
        self.nom_emetteur = hprim_nom_emetteur or ""
        self.code_recepteur = hprim_code_recepteur or ""
        self.nom_recepteur = hprim_nom_recepteur or ""
        self.option_comments_in_flags = option_comments_in_flags
        self.option_envoi_technicien = option_envoi_technicien
        self.option_envoi_sample_id = option_envoi_sample_id
        self.option_ipp_executant = option_ipp_executant
        self.option_envoyer_fic = option_envoyer_fic
        self.option_identification_demandeur = option_identification_demandeur
        self.option_annulation_sans_ipp_age_max = option_annulation_sans_ipp_age_max
        self.option_envoi_commentaire_patient = option_envoi_commentaire_patient
        self.option_envoi_code_groupe = option_envoi_code_groupe
        self.option_bioserveur = kwargs.pop("option_bioserveur", False)
        super(Driver, self).__init__(**kwargs)

    def to_string(self, message, *args, **kwargs):
        message.end.nombre_segment_patient = len(message.patients)
        message.end.nombre_segment = len(message.get_segments())
        return super(Driver, self).to_string(message, *args, **kwargs)

    def create(self, **kwargs):
        message = self.SEGMENT_CLASS_MAP["H"](type_segment="H", **kwargs)
        message.end = self.SEGMENT_CLASS_MAP["L"](type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context not in ("result", "order"):
            raise NotImplementedError("context %s" % (worklist.context,))

        msg = self.create(
            date=context["now"],
            type_message="ORU" if worklist.context == "result" else "ORM",
            nom_fichier=context.get("filename"),
        )
        msg.emetteur.code = worklist.sender_id or self.code_emetteur
        msg.emetteur.nom = worklist.sender_name or self.nom_emetteur
        msg.recepteur.code = worklist.receiver_id or self.code_recepteur
        msg.recepteur.nom = worklist.receiver_name or self.nom_recepteur

        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", "add": "A", "new": "N"},
        ):
            if worklist.context == "result" and patient.lab_patient_id:
                patient_demandeur = patient.lab_patient_id
            else:
                patient_demandeur = patient.patient_id

            seg_p = msg.patients.add(
                no_adm_cli=patient.alternate_patient_id,
                date_nai=patient.birthdate,
                sexe=patient.sex,
                nomjf=patient.maiden_name,
                date_entree=patient.admission_date,
                date_sortie=patient.admission_end_date,
                poids=patient.weight,
                taille=patient.height,
                rang_naissance=patient.birth_row,
            )
            seg_p.date_deces = patient.deathdate
            if self.option_ipp_executant:
                seg_p.patient_executant = patient_demandeur
            else:
                seg_p.patient_demandeur = patient_demandeur

            if patient.phone:
                if patient.phone == "T":
                    # Spécifique Clarilab
                    seg_p.set_raw("telephone", "~T")
                else:
                    seg_p.telephone = [p for p in patient.phone.split(";") if p]
            if patient.medications:
                seg_p.traitement_medical = patient.medications
            seg_p.nom.nom = patient.last_name
            seg_p.nom.prenom = patient.first_name
            seg_p.nom.civilite = patient.title
            seg_p.stat_adm = patient.admission_status
            seg_p.localisation.lit = patient.bed
            seg_p.localisation.chambre = patient.room
            seg_p.localisation.corr = patient.location
            seg_p.adresse.ville = patient.city
            seg_p.adresse.pays = patient.country
            seg_p.adresse.code_postal = patient.postal_code
            if patient.address:
                if "~" in patient.address:
                    seg_p.adresse.ligne1, seg_p.adresse.ligne2 = patient.address.split("~", 1)
                else:
                    seg_p.adresse.ligne1 = patient.address
            # envoi du commentaire patient si l'option
            # "option_envoi_commentaire_patient" est activée
            if self.option_envoi_commentaire_patient:
                comments_patient = []
                if patient.patient_comment1:
                    comments_patient.append(patient.patient_comment1)
                if patient.patient_comment2:
                    comments_patient.append(patient.patient_comment2)
                if comments_patient:
                    seg_p.commentaires.add(texte=comments_patient)
            for physician in patient.physicians:
                if physician.type == "attending":
                    med = seg_p.medecins.add()
                    med.code = physician.code
                    med.nom.nom = physician.last_name
                    med.nom.prenom = physician.first_name
                    med.type = physician.codification
                    med.nom.civilite = physician.title
                    med.nom.sobriquet = physician.alias
            for sample, action, priority in patient.samples:
                if not msg.emetteur.code:
                    msg.emetteur.code = sample.lab
                seg_o = seg_p.orders.add(
                    tube_demandeur=sample.sample_id,
                    no_demandeur=sample.sample_id if self.option_envoi_sample_id else sample.request_id or sample.sample_id,
                    priorite=priority or "R",
                    action=action or "N",
                    date_demande=sample.request_date,
                    date_prel=sample.collection_date,
                    date_fin_prel=sample.collection_end_date,
                )
                if worklist.context == "result":
                    if self.option_bioserveur:
                        # Bioserveur limite la réception du tube executant
                        # et du numéro executant à 23 carractères.
                        # Ils ignorent le "tube executant", donc on ne le précise pas.
                        seg_o.tube_demandeur = ''
                        seg_o.tube_executant = ''
                        seg_o.no_executant = seg_o.no_demandeur
                    else:
                        seg_o.tube_executant = sample.exe_sample_id or sample.sample_id
                        seg_o.no_executant = (
                            sample.exe_request_id or
                            sample.exe_sample_id or
                            sample.request_id or
                            sample.sample_id
                        )
                if sample.services or sample.physicians:
                    presc = seg_o.prescripteur.add()
                    for service in sample.services:
                        if service:
                            presc.code_service = service.code
                            presc.service.nom = service.name
                    for physician in sample.physicians:
                        if physician.type == "prescriber":
                            presc.code_medecin = physician.code
                            presc.medecin.nom = physician.last_name
                            if physician.phone:
                                seg_o.tel_prescrip = physician.phone.split(";")
                        elif physician.type == "sampler":
                            seg_o.preleveur.code = physician.code
                            seg_o.preleveur.nom.nom = physician.last_name
                            seg_o.preleveur.nom.prenom = physician.first_name
                            seg_o.preleveur.type = physician.codification
                            seg_o.preleveur.nom.civilite = physician.title
                            seg_o.preleveur.nom.sobriquet = physician.alias

                if sample.transcriptionist:
                    seg_o.ope_saisie.code = sample.transcriptionist
                combined_orders_results = []
                for o in sample.orders:
                    if o.code in [x["code"] for x in combined_orders_results]:
                        logger.info("code %s already present: do not add.", o.code)
                        continue
                    combined_orders_results.append({
                        "code": o.code,
                        "label": o.label,
                        "alt_code": o.alt_code,
                        "alt_label": o.alt_label,
                    })
                for r in sample.results:
                    if r.code in [x["code"] for x in combined_orders_results]:
                        logger.info("code %s already present: do not add.", r.code)
                        continue
                    combined_orders_results.append({
                        "code": r.code,
                        "label": r.label,
                        "alt_code": r.alt_code,
                        "alt_label": r.alt_label,
                    })
                for orderdict in combined_orders_results:
                    test = seg_o.prescription.add()
                    test.code1 = orderdict["code"]
                    if orderdict["label"]:
                        test.label1 = orderdict["label"]
                    if orderdict["alt_code"]:
                        test.code2 = orderdict["alt_code"]
                    if orderdict["alt_label"]:
                        test.label2 = orderdict["alt_label"]

                for result in sample.results:
                    if worklist.context == "order" and not result.anterior:
                        date = None
                    else:
                        date = result.date
                    valideur = None
                    if not self.option_envoi_technicien or not result.operator or result.validator != 'auto':
                        valideur = result.validator
                    else:
                        valideur = result.operator

                    def do_seg_r():
                        seg_r = seg_o.results.add(
                            type_resu=result.type,
                            code=result.code,
                            sous_test=result.sub_code,
                            date=date,
                            indic_norm=result.flags,
                            stat_resu=result.status,
                            analyseur=result.instrument,
                            valideur=valideur,
                            resultat_controle=result.resultat_controle,
                        )
                        seg_r.unite.code1 = result.unit
                        if self.option_bioserveur:
                            # Bioserveur nécessite de recevoir le code examen
                            # dans le secteur technique (code batterie)
                            seg_r.secteur_tech.code_batterie = result.code
                        return seg_r

                    if isinstance(result.value, basestring):
                        values = [{"value": v} for v in result.value.split(";")]
                    elif isinstance(result.value, dict):
                        values = [result.value]
                    elif isinstance(result.value, list):
                        values = result.value
                    else:
                        raise NotImplementedError

                    seg_r = None

                    multiple_clarilab = None
                    if values and len(values[0].get("antibiogram", [])):
                        multiple_clarilab = []
                        for a in values[0]["antibiogram"]:
                            multiple_clarilab.append((a["antibiotic"], a["mic"], a["sensitivity"]))
                    elif values and len(values[0].get("multiple_clarilab", [])):
                        multiple_clarilab = values[0]["multiple_clarilab"]

                    if multiple_clarilab:
                        # BEGIN INCANTATION VAUDOU

                        # Spéficique Clarilab
                        # AECO~ATB ESCCOL~L/A_VAM\8\I^ANTIBIO\CMI\SENSI^...
                        # BACT+~Culture~L/#\3C\G_STAAUR\#\#\#\#\#\#\#\#\#\#\#
                        assert len(values) == 1, values
                        data = self.DEFAULT_SEPARATORS[1].join([
                            values[0]["value"], values[0]["label"], values[0]["codification"]
                        ])
                        data += "/"
                        lines = []
                        for m in multiple_clarilab:
                            if isinstance(m, (list, tuple)):
                                lines.append(self.DEFAULT_SEPARATORS[3].join(m))
                            else:
                                lines.append(m)
                        # ATTENTION: C'est '^' en dur dans le code de clarilab
                        # (sgdl/hprim/resultat.py)
                        data += "^".join(lines)

                        seg_r = do_seg_r()
                        seg_r.set_raw("resu", data)

                        # END INCANTATION VAUDOU
                    else:
                        seg_r = do_seg_r()
                        for value in values:
                            resu = seg_r.resu.add()
                            resu.code1 = value["value"]
                            resu.label1 = value.get("label")
                            resu.codification1 = value.get("codification")

                    if (
                        seg_r is not None and
                        result.type == "GC" and
                        all([not v.get("label") for v in values])
                    ):
                        # Graphe à pas fixe de type y^y^y Dans
                        # ce cas il faut renseigner les champs
                        # pas et origine
                        seg_r.gc_pas = "1"
                        seg_r.gc_origine = "0"

                    for comment in result.comments:
                        if seg_r is not None:
                            seg_r.commentaires.add(origine="L", texte=[comment])

        if worklist.context == "result":
            msg = self.clean_result_msg(msg)

        if msg.patients:
            return msg

    def clean_result_msg(self, msg):
        # Ne garde que les segment OBR qui ont des OBX associés
        for seg_p in msg.patients:
            for seg_o in seg_p.orders[:]:
                if not seg_o.results:
                    seg_p.orders.remove(seg_o)

        for seg_p in msg.patients[:]:
            if not seg_p.orders:
                msg.patients.remove(seg_p)

        return msg

    def ignore_seg_p(self, seg_p):
        for seg_obr in seg_p.orders:
            for seg_obx in seg_obr.results:
                if seg_obx.date is None and seg_obx._raw_date:
                    logger.error("Date OBX mal formée '%s' on ignore le segment P", seg_obx._raw_date)
                    return True
        return False

    def add_result(self, worklist, sample, seg_obx):
        if worklist.context == "result" and seg_obx.stat_resu == "X":
            return

        if (
            worklist.context == "order" and
            seg_obx.date is not None
        ):
            anterior = True
        else:
            anterior = False

        sep = {
            "TX": "\n",
        }.get(seg_obx.type_resu, ";")

        if seg_obx.type_resu == "FIC":
            # 10.6.1 -> Nom de l'application éméttrice
            # 10.6.2 -> Identification du fichier
            # 10.6.3 -> Type de fichier
            value = sep.join([r.label1 for r in seg_obx.resu])
            label = seg_obx.code2
        else:
            value = sep.join([r.code1 for r in seg_obx.resu])
            label = sep.join([r.label1 for r in seg_obx.resu])

        flags = []
        if self.option_comments_in_flags:
            for seg_c in seg_obx.commentaires:
                flags.extend(seg_c.texte)
        if seg_obx.type_resu == "FIC" and seg_obx.sous_test:
            code = "%s_%s" % (seg_obx.code, seg_obx.sous_test)
        else:
            code = seg_obx.code
        if seg_obx.droit_acces:
            instrument = seg_obx.droit_acces
        else:
            instrument = ""
        result = sample.results.get_or_create(
            type=seg_obx.type_resu,
            code=code,
            label=seg_obx.libelle,
            sub_code=seg_obx.sous_test,
            date=seg_obx.date,
            value=value,
            value_label=label,
            flags=flags,
            anterior=anterior,
            unit=seg_obx.unite.code1,
            instrument=instrument,
        )
        if seg_obx.valideur:
            result.validator = seg_obx.valideur
        if anterior:
            result.request_id = seg_obx.no_demande
            result.request_date = seg_obx.date_demande

    def decode_message(self, message):
        if message.type_message not in ("ORU", "ORM"):
            raise NotImplementedError("type message %s" % (message.type_message,))

        worklist = Worklist(
            context="result" if message.type_message == "ORU" else "order",
            sender_id=message.emetteur.code,
            receiver_id=message.recepteur.code,
        )
        for seg_p in message.patients:
            if self.ignore_seg_p(seg_p):
                continue
            for seg_obr in seg_p.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_obr.tube_demandeur,
                    request_id=seg_obr.no_demandeur,
                    exe_sample_id=seg_obr.tube_executant,
                    exe_request_id=seg_obr.no_executant,
                    patient_id=seg_p.patient_demandeur or seg_p.patient_executant or seg_obr.no_demandeur,
                )
                if seg_p.telephone == [["", "T"]]:
                    # À telephoner clarilab...
                    phone = ""
                else:
                    phone = ";".join([p for p in seg_p.telephone if p])
                sample.update(
                    alternate_patient_id=seg_p.no_adm_cli,
                    last_name=seg_p.nom.nom,
                    first_name=seg_p.nom.prenom,
                    title=seg_p.nom.civilite,
                    maiden_name=seg_p.nomjf,
                    birthdate=seg_p.date_nai,
                    birth_row=seg_p.rang_naissance,
                    sex=seg_p.sexe if seg_p.sexe in ("M", "F") else "U",
                    request_date=(
                        seg_obr.date_demande or
                        seg_obr.date_prel or
                        message.date),
                    type="patient",
                    diagnostic=";".join([e.code1 for e in seg_p.diagnostic_patient]),
                    location=seg_p.localisation.corr,
                    room=seg_p.localisation.chambre,
                    bed=seg_p.localisation.lit,
                    collection_date=seg_obr.date_prel,
                    collection_end_date=seg_obr.date_fin_prel,
                    admission_date=seg_p.date_entree,
                    admission_end_date=seg_p.date_sortie,
                    height=seg_p.taille,
                    phone=phone,
                    transcriptionist=seg_obr.ope_saisie.code,
                    city=seg_p.adresse.ville,
                    postal_code=seg_p.adresse.code_postal,
                    country=seg_p.adresse.pays,
                    address="~".join([
                        l for l in [seg_p.adresse.ligne1, seg_p.adresse.ligne2]
                        if l
                    ]),
                    admission_status=seg_p.stat_adm,
                    deathdate=seg_p.date_deces,
                    physiologie=self.get_physiologie(seg_p),
                )
                if self.option_identification_demandeur:
                    sample.request_info = seg_obr.lib1_dem
                if seg_p.poids:
                    sample.weight = unicode(seg_p.poids)
                for service in seg_obr.get_services():
                    if seg_obr.prescripteur and seg_obr.prescripteur[0].code_service and message.version == "H2.4":
                        continue
                    else:
                        sample.services.get_or_create(**service)
                for type, segments in (
                    ("patient", seg_p.commentaires),
                    ("sample", seg_obr.commentaires),
                ):
                    for seg_c in segments:
                        for item in seg_c.texte:
                            sample.comments.get_or_create(type=type, text=item)
                for item in seg_obr.get_comments():
                    sample.comments.get_or_create(type="sample", text=item)
                for item in [
                    seg_p.traite_lc1, seg_p.traite_lc2,
                    seg_p.comment_dem1, seg_p.comment_dem2,
                ]:
                    if item:
                        sample.comments.get_or_create(type="patient", text=item)
                for item in seg_p.traitement_medical:
                    sample.medications.get_or_create(item)
                for med in seg_p.medecins:
                    if med.code not in [p.code for p in sample.physicians]:
                        sample.physicians.get_or_create(
                            code=med.code,
                            last_name=med.nom.nom,
                            first_name=med.nom.prenom,
                            type="attending",
                            codification=med.type,
                            title=med.nom.civilite,
                            alias=med.nom.sobriquet,
                        )
                self.get_medecin_prescripteur(sample, seg_obr)

                if seg_obr.preleveur.code:
                    sample.physicians.get_or_create(
                        type="sampler",
                        code=seg_obr.preleveur.code,
                        last_name=seg_obr.preleveur.nom.nom,
                        first_name=seg_obr.preleveur.nom.prenom,
                        codification=seg_obr.preleveur.type,
                        title=seg_obr.preleveur.nom.civilite,
                        alias=seg_obr.preleveur.nom.sobriquet,
                    )
                if worklist.context == "order":
                    combined_orders_results = []
                    for p in seg_obr.prescription:
                        if p.code1 in [x["code"] for x in combined_orders_results]:
                            logger.info("code %s already present: do not add.", p.code1)
                            continue
                        combined_orders_results.append({
                            "code": p.code1,
                            "label": p.label1,
                            "alt_code": p.code2,
                            "alt_label": p.label2,
                        })
                    for r in [r for r in seg_obr.results if r.date is None]:
                        if r.code in [x["code"] for x in combined_orders_results]:
                            logger.info("code %s already present: do not add.", r.code)
                            continue
                        combined_orders_results.append({
                            "code": r.code,
                            "label": r.label if hasattr(r, "label") else None,
                            "alt_code": r.alt_code if hasattr(r, "alt_code") else None,
                            "alt_label": r.alt_label if hasattr(r, "alt_label") else None,
                        })
                    for orderd in combined_orders_results:
                        sampleorder = sample.orders.get_or_create(
                            code=orderd["code"],
                            priority=seg_obr.priorite if seg_obr.priorite else "R",
                            action=seg_obr.get_action(),
                        )
                        if orderd.get("label"):
                            sampleorder.label = orderd["label"]
                        if orderd.get("alt_code"):
                            sampleorder.alt_code = orderd["alt_code"]
                        if orderd.get("alt_label"):
                            sampleorder.alt_label = orderd["alt_label"]
                self.decode_obr(seg_obr, worklist, sample)
        return worklist

    def decode_obr(self, seg_obr, worklist, sample):
        for seg_obx in seg_obr.results:
            self.add_result(worklist, sample, seg_obx)

    def get_physiologie(self, seg_p):
        return ""

    def get_medecin_prescripteur(self, sample, seg_obr):
        for prescripteur in seg_obr.prescripteur:
            if (
                prescripteur.code_medecin and
                prescripteur.code_medecin not in [p.code for p in sample.physicians]
            ):
                sample.physicians.get_or_create(
                    code=prescripteur.code_medecin,
                    last_name=prescripteur.medecin.nom,
                    codification=prescripteur.type_medecin,
                    type="prescriber",
                    phone=";".join(seg_obr.tel_prescrip),
                )
