# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.splitted import Splitted, String, Date
from driverng import MessageV2


class InterQCBase(Splitted):
        SEPARATOR = "|"


class InterQCResult(InterQCBase):
    FIELDS = (
        String("instrument_id"),
        String("lot"),
        String("level"),
        String("code_instrument"),
        Date("date", fmt_date="%Y%m%d"),
        Date("time", fmt_date="%H%M"),
        String("result"),
        String("units"),
        String("code_lis"),
        # if not use, must be set to 0
        String("analyte_id"),
        String("lot_reagent"),
        String("name_control"),
    )


class InterQC(list):

    def add(self, **kwargs):
        obj = InterQCResult(**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not line:
                continue
            self.append(InterQCResult().parse(line))
        return self

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "interqc",
        "label": "InterQC INGEN",
    }

    OPTIONS = [MessageV2.Option("customer_id", label="Customer ID")]

    def __init__(self, customer_id="", **kwargs):
        self.customer_id = customer_id
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return InterQC(**kwargs)

    def parse(self, data):
        return InterQC().parse(data)

    def to_string(self, message):
        return message.to_string()

    def encode_worklist(self, worklist, context):
        msg = self.create()
        # Uniquement les contextes de type result
        if worklist.context != "result":
            return None
        for sample in worklist.samples:
            # Uniquement les samples de type control
            if sample.type != "control":
                continue
            for result in sample.results:
                instrument_id = "/%s/%s" % (
                    result.instrument,
                    self.customer_id,
                )
                reagent = ""
                for lot in result.lots:
                    if lot.type == "reagent":
                        reagent = lot.lot
                        break
                if sample.control_name:
                    nom_lot = sample.control_name
                else:
                    nom_lot = sample.sample_id
                msg.add(
                    instrument_id=instrument_id,
                    lot=sample.sample_id,
                    level=sample.control_level,
                    code_instrument=result.code,
                    date=result.date,
                    time=result.date,
                    result=result.value,
                    units=result.unit,
                    code_lis=result.code,
                    analyte_id="0",
                    lot_reagent=reagent,
                    name_control=nom_lot,
                )
        if msg:
            return msg
        else:
            return None
