# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2, Worklist
from driverng.fields import String


class KpcrAdBase(object):
    FIELDS = ()

    def __init__(self, **kwargs):
        for field in self.FIELDS:
            setattr(self, field.name, kwargs.get(field.name, None))
        super(KpcrAdBase, self).__init__()

    def parse(self, data=None):
        fields = self.FIELDS
        for idx, elt in enumerate(data.split("|")):
            if not elt:
                continue
            try:
                field = fields[idx]
            except KeyError:
                pass
            else:
                value = field.to_python(elt)
                setattr(self, field.name, value)

    def to_string(self):
        ligne = []
        for field in self.FIELDS:
            ligne.append("%s" % field.to_string(getattr(self, field.name)))
        return "%s" % ("|".join(ligne))


class KpcrAdResult(KpcrAdBase):
    FIELDS = (
        String("well"),
        String("sample_id"),
        String("channel"),
        String("well_type"),
        String("quantity"),
        String("notes"),
        String("ct"),
        String("well_comment"),
    )


class KpcrAdMultipleMessage(list):
    _MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: KpcrAdResult,
    }

    def add(self, message_type=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[message_type](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not len(line):
                continue
            splitted = line.split("|")
            # Un résultat patient ou un QC est composé de 8 champs
            if len(splitted) == 8:
                message_type = MessageV2.RESULT
                obj = self.add(message_type=message_type)
                obj.parse(line)

    def to_string(self):
        return "\r\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {
        "code": "kpcr_ad",
        "label": "KPCR - Amplificateur/Détecteur",
    }

    MESSAGE_TYPE_MAP = {
        MessageV2.RESULT: KpcrAdResult,
    }

    def create(self, message_type, **kwargs):
        return KpcrAdMultipleMessage()

    def to_string(self, message):
        return message.to_string()

    def parse(self, data):
        obj = KpcrAdMultipleMessage()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        for m in message:
            if m.well_type == "Unknown":
                sample = worklist.get_or_create(
                    sample_id=m.sample_id,
                    type="patient",
                )
            elif m.well_type not in ('Unknown', 'Calibrator'):
                lot = "%s-%s" % (m.sample_id, m.channel,)
                sample = worklist.get_or_create(
                    sample_id=lot,
                    control_lot=lot,
                    type="control",
                )
            # resultat
            sample.results.add(
                code="%s%s" % (m.well, m.channel,),
                value=m.quantity,
            )
        return worklist
