# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimMultiple,
    HprimList,
    HprimDate,
    HprimForeign,
)


class StarexpSegmentH(HeaderSegment):

    sender_id = HprimString((5, 1), default='99')
    sender_name = HprimString((5, 2))
    processing_id = HprimString(12, default='P')
    version = HprimString(13, default="LIS2-A2")
    date = HprimDate(14)
    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign(
        "L", "header", one2one=True, required=True
    )


class StarexpSegmentP(Segment):

    patient_id = HprimString(3, max_length=11)
    last_name = HprimString((6, 1), max_length=40)
    first_name = HprimString((6, 2), max_length=40)
    title = HprimString((6, 5), max_length=5)
    nomjf = HprimString(7)
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    # M/F/U
    sex = HprimString(9)
    adress_line1 = HprimString((11, 1))
    adress_line2 = HprimString((11, 2))
    adress_city = HprimString((11, 3))
    adress_zip_code = HprimString((11, 5))
    physician_id = HprimString((14, 1))
    physician_name = HprimString((14, 2))
    height = HprimString(17)
    weight = HprimString(18)
    diagnosis = HprimString(19)
    medication = HprimString(20)
    comment1 = HprimString(22)
    comment2 = HprimString(23)
    service = HprimString((26, 3), max_length=10)
    orders = HprimForeign("O", "patient")


class StarexpSegmentQ(Segment):

    tube = HprimString((3, 2), max_length=16)


class StarexpSegmentO(Segment):

    specimen_id = HprimString((3, 1))
    rack = HprimString((3, 2))
    position = HprimString((3, 3))
    qc_lot_no = HprimString((3, 4))
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        code=HprimString(4),
        name=HprimString(5),
    ))

    # S: Stat
    # R: Routine
    priority = HprimString(6, default='R')
    collection_date = HprimDate(8)

    action_code = HprimString(12, default='N')
    physician_id = HprimString((17, 1))
    physician_name = HprimString((17, 2))
    service_id = HprimString((17, 4))
    service_name = HprimString((17, 5))
    report_type = HprimString(26)
    operator = HprimString(33)

    results = HprimForeign("R", "order")


class StarexpSegmentR(Segment):

    test = HprimString((3, 4))
    test_name = HprimString((3, 5))
    test_status = HprimString((3, 7))
    value = HprimString(4)
    units = HprimString(5)
    flag = HprimString(7)
    # F: Final
    status = HprimString(9)
    operator = HprimString(11)
    date = HprimDate(13)
    instrument = HprimString(14)
    manufacturers = HprimForeign("M", "result")


class StarexpSegmentM(Segment):
    # M|1|XXXXXX

    errors = HprimString(3)
    alarms = HprimString(4)
    well = HprimString(5)
    reagenta_id = HprimString((6, 1))
    reagenta_name = HprimString((6, 2))
    reagenta_lot = HprimString((6, 3))
    reagentb_id = HprimString((7, 1))
    reagentb_name = HprimString((7, 2))
    reagentb_lot = HprimString((7, 3))
    reagentc_id = HprimString((8, 1))
    reagentc_name = HprimString((8, 2))
    reagentc_lot = HprimString((8, 3))
    reagentd_id = HprimString((9, 1))
    reagentd_name = HprimString((9, 2))
    reagentd_lot = HprimString((9, 3))
    diluent_id = HprimString((10, 1))
    diluent_name = HprimString((10, 2))
    diluent_lot = HprimString((10, 3))
    cuvette = HprimString(11)
    cleaner_id = HprimString((12, 1))
    cleaner_name = HprimString((12, 2))
    cleaner_lot = HprimString((12, 3))
    desorba_id = HprimString((13, 1))
    desorba_name = HprimString((13, 2))
    desorba_lot = HprimString((13, 3))
    desorbb_id = HprimString((14, 1))
    desorbb_name = HprimString((14, 2))
    desorbb_lot = HprimString((14, 3))
    desorbc_id = HprimString((15, 1))
    desorbc_name = HprimString((15, 2))
    desorbc_lot = HprimString((15, 3))
    desorbd_id = HprimString((16, 1))
    desorbd_name = HprimString((16, 2))
    desorbd_lot = HprimString((16, 3))
    calib = HprimString(17)


class StarexpSegmentC(Segment):
    # C|1|I|XXXXXX

    source = HprimString(3)
    commentaire = HprimString((4, 1))
    comment_long1 = HprimString((4, 2))
    comment_long2 = HprimString((4, 3))
    comment_long3 = HprimString((4, 4))
    comment_long4 = HprimString((4, 5))
    comment_long5 = HprimString((4, 6))


class StarexpSegmentL(Segment):

    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "starexp_astm",
        "label": "STA Coag Expert ASTM",
    }
    SEGMENT_CLASS_MAP = {
        "H": StarexpSegmentH,
        "P": StarexpSegmentP,
        "O": StarexpSegmentO,
        "Q": StarexpSegmentQ,
        "R": StarexpSegmentR,
        "M": StarexpSegmentM,
        "C": StarexpSegmentC,
        "L": StarexpSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    def create(self, **kwargs):
        message = StarexpSegmentH(type_segment="H", **kwargs)
        message.end = StarexpSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(
            date=context["now"],
        )
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": "C", worklist.ELSE: "N"},
            priority={"S": "S", worklist.ELSE: "R"},
        ):
            if patient.services:
                service_code = patient.services[0].code
            else:
                service_code = ''
            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                height=patient.height,
                weight=patient.weight,
                service=service_code,

            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    specimen_id=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    collection_date=sample.request_date,
                    service_id=service_code,
                )
                for physician in sample.physicians:
                    seg_p.physician_id = physician.code
                    seg_p.physician_name = physician.last_name
                    seg_o.physician_id = physician.code
                    seg_o.physician_name = physician.last_name
                for order in sample.orders:
                    test = seg_o.tests.add()
                    test.code = order.code
                if action == "N":
                    for result in sample.results:
                        seg_o.results.add(
                            test=result.code,
                            value=result.value,
                            status="N",
                            date=result.date,
                        )

        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.tube
                )
            return worklist

        worklist = Worklist(context="result")
        for seg_p in message.patients:
            for seg_o in seg_p.orders:
                sample = worklist.get_or_create(
                    sample_id=seg_o.specimen_id,
                    type="control" if message.processing_id == "Q" else "patient",
                )
                if sample.type == "control":
                    sample.control_lot = "%s%s" % (
                        seg_o.specimen_id,
                        seg_o.qc_lot_no,
                    )
                sample.rack = "%s-%s" % (seg_o.rack, seg_o.position)
                for seg_r in seg_o.results:
                    # uniquement les resultats avec le statut F
                    if seg_r.status != "F":
                        continue
                    commentaires = []
                    alarmes = []
                    if seg_r.manufacturers:
                        if seg_r.manufacturers[0].alarms != "@":
                            alarmes = [seg_r.manufacturers[0].alarms]
                        if seg_r.manufacturers[0].errors != "A":
                            alarmes.append(seg_r.manufacturers[0].errors)
                    for seg_c in seg_r.commentaires:
                        commentaires.extend([c for c in [
                            seg_c.commentaire,
                            seg_c.comment_long1,
                            seg_c.comment_long2,
                            seg_c.comment_long3,
                            seg_c.comment_long4,
                            seg_c.comment_long5]
                            if c])
                    result = sample.results.add(
                        code=seg_r.test,
                        date=seg_r.date,
                        value=seg_r.value,
                        flags=alarmes,
                        instrument=seg_r.instrument,
                        comments=commentaires,
                    )
                    if sample.type == 'patient' and seg_r.operator:
                        result.operator = seg_r.operator
                    for seg_m in seg_r.manufacturers:
                        for key in "abcd":
                            lot = getattr(seg_m, "reagent%s_lot" % (key,))
                            name = getattr(seg_m, "reagent%s_name" % (key,))
                            if lot:
                                result.lots.add(type="reagent", lot=lot, name=name)
        return worklist
