# -*- coding: utf8 -*-
from __future__ import unicode_literals


from driverng.astm import (
    HprimMultiple,
    HprimString,
    HprimDate,
)
from driverng.drivers.hprim_spec import (
    Driver as Hprim,
    HprimSegmentH,
    HprimSegmentP,
    HprimSegmentOBR,
    HprimSegmentOBX,
)


class SynergieSegmentH(HprimSegmentH):
    version = HprimString((13, 1), default="A.2")


class SynergieSegmentP(HprimSegmentP):

    localisation = HprimMultiple(
        26,
        corr=HprimString(1),
        chambre=HprimString(2),
        lit=HprimString(3),
    )


class SynergieSegmentOBR(HprimSegmentOBR):

    service_code = HprimString((19, 1))
    service_name = HprimString((19, 2))

    def get_action(self):
        return {
            "C": "del",
            "S": "del",
            "A": "add",
        }.get(self.action, "new")

    def get_services(self):
        if self.service_code:
            return [{
                "code": self.service_code,
                "name": self.service_name,
            }]
        else:
            return []

    def get_comments(self):
        return [e for e in [
            self.renseignement_clinique
        ] if e]


class SynergieSegmentOBX(HprimSegmentOBX):

    date = HprimDate(13)
    sous_test = HprimString((5, 1))
    collection_conditions = HprimString((5, 2))


class Driver(Hprim):
    SPEC = {
        "label": "Synergie ASTM",
        "code": "synergie",
    }

    SEGMENT_CLASS_MAP = dict(Hprim.SEGMENT_CLASS_MAP.items() + {
        "H": SynergieSegmentH,
        "P": SynergieSegmentP,
        "OBR": SynergieSegmentOBR,
        "OBX": SynergieSegmentOBX,
    }.items())

    def add_result(self, worklist, sample, seg_obx):
        if worklist.context == "result" and seg_obx.stat_resu == "X":
            return

        anterior = False
        if (
            worklist.context == "order" and
            seg_obx.date is not None and
            seg_obx.stat_resu == "A"
        ):
            anterior = True
        else:
            anterior = False

        if seg_obx.type_resu == "FIC":
            # 10.6.1 -> Nom de l'application éméttrice
            # 10.6.2 -> Identification du fichier
            # 10.6.3 -> Type de fichier
            value = ";".join([r.label1 for r in seg_obx.resu])
        else:
            value = ";".join([r.code1 for r in seg_obx.resu])

        if worklist.context == "order" and not value:
            return

        result = sample.results.get_or_create(
            type=seg_obx.type_resu,
            code=seg_obx.code,
            anterior=anterior,
        )
        if result.value and value not in result.value.split(";"):
            result.value = ";".join(result.value.split(";") + [value])
        elif not result.value:
            result.value = value
        result.sub_code = seg_obx.sous_test
        result.date = seg_obx.date
        result.unit = seg_obx.unite.code1
