# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
from StringIO import StringIO

from driverng import Message


def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, **kwargs):
    # csv.py doesn't do Unicode; encode temporarily as UTF-8:
    csv_reader = csv.reader(utf_8_encoder(unicode_csv_data),
                            dialect=dialect, **kwargs)
    for row in csv_reader:
        # decode UTF-8 back to Unicode, cell by cell:
        yield [cell.decode('utf-8') for cell in row]


def utf_8_encoder(unicode_csv_data):
    for line in unicode_csv_data:
        yield line.encode('utf-8')


class Driver(Message):
    SPEC = {
        "code": "test_csv",
        "label": "Format de test CSV",
    }

    def parse(self, data):
        return unicode_csv_reader(StringIO(data), delimiter=b";")

    def get_message_type(self, message):
        return Message.ORDER

    def get_result_dict(self, message):
        result = {}
        for line in message:
            if not line:
                continue
            tube = line[1]
            if tube not in result:
                result[tube] = {"resultats": []}

            try:
                alarmes = line[8].split(",")
            except IndexError:
                alarmes = []

            try:
                commentaires = [line[9]]
            except IndexError:
                commentaires = []

            date = datetime.datetime.now()
            for fmt in ("%Y%m%d%H%M%S", "%Y%m%d%H%M"):
                try:
                    date = datetime.datetime.strptime(line[7], fmt)
                except (ValueError, KeyError):
                    pass

            result[tube]["resultats"].append({
                "test": line[5],
                "resu": line[6],
                "date": date,
                "alarmes": alarmes,
                "analyseur": line[2],
                "commentaires": commentaires,
                "connexion": line[0],
            })

        return result

    def get_qc_result_list(self, message):
        return []
