# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2
from driverng import Worklist
from driverng.fixed import FixedLengthFormat, String


class VtSyslabBase(FixedLengthFormat):
    pass


class VtSyslabResult(VtSyslabBase):
    FMT_DESCRIPTION = [
        String("id_message", 1),
        String("no_station", 2),
        String("sample_id", 8, right_justified=True),
        String("no_portoir", 2),
        String("position", 2),
        String("no_analyse_1", 2),
        String("resultat_1", 4, right_justified=True),
        String("no_analyse_2", 2),
        String("resultat_2", 4),
    ]


class Driver(MessageV2):
    SPEC = {
        "code": "vt_syslab",
        "label": "VT Protocole Syslab",
    }

    def create(self, message_type, **kwargs):
        if message_type == self.RESULT:
            return VtSyslabResult(**kwargs)
        else:
            raise NotImplementedError

    def parse(self, data):
        obj = VtSyslabResult()
        obj.parse(data)
        return obj

    def to_string(self, message):
        return message.to_string()

    def decode_message(self, message):
        worklist = Worklist(context="result")
        if message.id_message == "R":
            sample = worklist.get_or_create(
                type="patient",
                sample_id=message.sample_id,
            )
        else:
            raise NotImplementedError

        # resultat VS 1er Heure
        if message.no_analyse_1 and message.resultat_1:
            sample.results.add(
                code=message.no_analyse_1,
                value=message.resultat_1,
            )
        # resultat VS 2e Heure
        if message.no_analyse_2 and message.resultat_2:
            sample.results.add(
                code=message.no_analyse_2,
                value=message.resultat_2,
            )
        return worklist
