# -*- coding: utf8 -*-
from __future__ import unicode_literals


from driverng.utils import get_date, format_date


class Field(object):
    def __init__(self, name):
        self.name = name
        super(Field, self).__init__()

    def to_python(self, value):
        raise NotImplementedError

    def to_string(self, value):
        raise NotImplementedError


class String(Field):
    def __init__(self, name, max_length=None):
        self.max_length = max_length
        super(String, self).__init__(name)

    def to_python(self, value):
        return value.strip()

    def to_string(self, value):
        if value is None:
            return ""
        if self.max_length:
            return value[:self.max_length]
        else:
            return value


class Date(Field):
    def __init__(self, name, fmt):
        self.fmt = fmt
        super(Date, self).__init__(name)

    def to_python(self, value):
        return get_date(value, fmt=self.fmt)

    def to_string(self, value):
        return format_date(value, fmt=self.fmt)
