# -*- coding: utf8 -*-

from __future__ import unicode_literals
import decimal
from driverng.astm.fields import (
    BaseHprimField,
    HprimMultiple,
    HprimDate,
    HprimString as HL7String,
    HprimCE as HL7CE,
    HprimForeign as HL7Foreign,
    HprimList as HL7List,
)

__all__ = [
    "HL7String", "HL7CE", "HL7Foreign", "HL7List",
    "HL7HD", "HL7DT", "HL7DTM", "HL7IS", "HL7NM", "HL7MSG",
    "HL7Money", "HL7CompositePrice", "HL7CompositeQuantity", "HL7RP",
]


class HL7IS(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "value": HL7String(1),
            "label": HL7String(2),
        })
        super(HL7IS, self).__init__(*position, **kwargs)


class HL7HD(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "nsid": HL7IS(1),
            "uid": HL7String(2),
            "uidtype": HL7String(3),
        })
        super(HL7HD, self).__init__(*position, **kwargs)


class HL7DT(HprimDate):
    HL7_DATE_FMT = "%Y%m%d"

    def __init__(self, *position, **kwargs):
        kwargs.update({'fmt_date': self.HL7_DATE_FMT})
        super(HL7DT, self).__init__(*position, **kwargs)


class HL7DTM(HprimDate):
    HL7_DATE_FMT = "%Y%m%d%H%M%S"


class HL7NM(BaseHprimField):
    def to_python(self, value):
        # IN HL7, NM is a double, use Decimal for precision
        if not value:
            return None
        try:
            return decimal.Decimal(value)
        except (ValueError, decimal.InvalidOperation):
            pass


class HL7MSG(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "message_type": HL7String(1, obligatoire=True),
            "event_type": HL7String(2, obligatoire=True),
            "structure": HL7String(3, obligatoire=True),
        })
        super(HL7MSG, self).__init__(*position, **kwargs)


class HL7Money(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "quantity": HL7NM(1),
            # ISO-4217
            "currency": HL7String(2, default="EUR"),
            "other": HL7String(3, default="BLA"),
        })
        super(HL7Money, self).__init__(*position, **kwargs)


class HL7PT(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "production": HL7String(1),
            "test": HL7String(2),
        })
        super(HL7PT, self).__init__(*position, **kwargs)


class HL7CompositePrice(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "price": HL7Money(1, obligatoire=True),
        })
        super(HL7CompositePrice, self).__init__(*position, **kwargs)


class HL7RP(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "pointer": HL7String(1),
            "type": HL7String(2, default="AP"),
            "subtype": HL7String(3),
        })
        super(HL7RP, self).__init__(*position, **kwargs)


class HL7CompositeQuantity(HprimMultiple):
    def __init__(self, *position, **kwargs):
        kwargs.update({
            "quantity": HL7NM(1),
            "unit": HL7CE(2),
        })
        super(HL7CompositeQuantity, self).__init__(*position, **kwargs)
