# -*- coding: utf-8 -*-

from driverng.astm.segment import AbstractSegment as Segment
from driverng.astm.parser import RecursiveList
from driverng.hl7.fields import (
    HL7CE,
    HL7HD,
    HL7DTM,
    HL7MSG,
    HL7String,
    HL7NM,
    HL7Foreign,
)


class HeaderSegment(Segment):
    def to_string(self, separators=None):
        if separators is None:
            separators = RecursiveList.DEFAULT_SEPARATORS

        self.rang_segment = ""
        return "%s%s%s" % (
            self.type_segment,
            separators,
            super(HeaderSegment, self).to_string(separators)[
                len(self.type_segment) + 1:],
        )


class HL7SegmentMSH(HeaderSegment):
    """Message Header Segment."""
    # Sending Application
    emetteur_app = HL7HD(3, obligatoire=True)

    # Sending Facility
    # finesstype:
    #  entité juridique: M
    #  entité géographique: N
    emetteur = HL7HD(4, obligatoire=True)

    # Receiving Application
    #   if not found: UNKNOWN
    recepteur_app = HL7HD(5)

    # Receiving Facility
    #   if not found: UNKNOWN
    recepteur = HL7HD(6)

    # Date/Type of Message
    date = HL7DTM(7, obligatoire=True)

    # Type MSG
    type_message = HL7MSG(9, obligatoire=True)

    # Identifiant unique du message.
    message_control_id = HL7String(10, obligatoire=True, max_length=20)

    # Processing Id
    # D: Debugging P: Production T: Training
    mode_traitement = HL7String(11, obligatoire=True, max_length=3)

    # Version du standard HL7
    version_id = HL7String(12,
                           obligatoire=True,
                           max_length=60,
                           default="2.5")

    # ISO 3166 country code
    # Type ID
    country_code = HL7String(17,
                             obligatoire=True,
                             max_length=3,
                             default='FRA')

    # Type ID
    encoding = HL7String(18,
                         obligatoire=True,
                         max_length=16,
                         default='8859/15')

    # Type CE ?
    principal_language_message = HL7CE(19, max_length=250)

    message_profile_identifier = HL7String(21, max_length=427)

    # Champs interdits en france
    security = HL7String(8, max_length=40)
    sequence_number = HL7NM(13, max_length=15)
    continuation_pointer = HL7String(14, max_length=180)
    accept_ack_type = HL7String(15)
    app_ack_type = HL7String(16)
    alternate_character_set = HL7String(20, max_length=20)

    mfi = HL7Foreign("MFI", "msh")
