# -*- coding: utf -*-
from __future__ import unicode_literals

import time
import datetime

from driverng.protocol.mllp import MLLPProtocol


class AtellicaProtocol(MLLPProtocol):
    HL7_ACK = b"""MSH|^~\&|%(sender_id)s|\"\"|%(receiver_id)s|\"\"|%(date)s||%(message_type)s|%(control_id)s|P|2.5.1|||NE|AL||UNICODE UTF-8|||%(profile_message)s\rMSA|AA|%(ack_id)s|\r"""
    HL7_QACK = b"""MSH|^~\&|%(sender_id)s|\"\"|%(receiver_id)s|\"\"|%(date)s||%(message_type)s|%(control_id)s|P|2.5.1|||NE|AL||UNICODE UTF-8|||%(profile_message)s\rMSA|AA|%(ack_id)s|\rQAK||OK|WOS^Work Order Step^IHELAW\rQPD|WOS^Work Order Step^IHELAW|"""

    def __init__(self, **kwargs):
        super(AtellicaProtocol, self).__init__(**kwargs)

    def get_ack_message(self, payload, date=None, control_id=None):
        splitted = payload.splitlines()[0].split("|")

        date = date or datetime.datetime.now()
        type_message = splitted[8]
        control_id = control_id or int(time.time() * 1000)
        context = {
            "date": date.strftime("%Y%m%d%H%M%S"),
            "control_id": control_id,
            "sender_id": splitted[4],
            "receiver_id": splitted[2],
            "ack_id": splitted[9],
            "profile_message": splitted[20][:10],
        }
        if self.is_automate:
            context["message_type"] = b"OML^033^OML_033"
        else:
            if type_message.startswith("QBP"):
                context["message_type"] = b"RSP^K11^RSP_K11"
                return AtellicaProtocol.HL7_QACK % context
            else:
                context["message_type"] = b"ACK^R22^ACK"
                return AtellicaProtocol.HL7_ACK % context
