# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time

from driverng.utils import ACK, NACK, DEL, CAN, byte2log
from driverng.protocol.acknack import ACKNACKProtocol


class Cd3200Protocol(ACKNACKProtocol):

    def __init__(self,
                 checksum_type=ACKNACKProtocol.CHECKSUM_ASTM,
                 **kwargs):
        super(Cd3200Protocol, self).__init__(
            checksum_type=checksum_type, **kwargs)

    def send_trames(self, trame):
        while self.tries < self.max_tries:
            self.tries += 1
            self.write(trame)
            data = self.read(timeout=10)
            if not data or data[0] not in (ACK, DEL, NACK, CAN):
                self.logger.info(
                    "Unexpected data '%s', wait 3 seconds and retry",
                    byte2log(data),
                )
                time.sleep(3)
                return False

            if data[0] in (ACK, DEL):
                if data[0] == DEL:
                    self.logger.warning("Already in the Work List: skip")
                return True

            if data[0] == CAN:
                self.logger.error("Worklist Full")
                return False

        return False
