# -*- coding: utf8 -*-
from __future__ import unicode_literals

import random

from driverng.utils import byte2log, ENQ, EOT, ACK, NACK
from driverng.protocol.astm import AstmProtocol


class DmAstmProtocol(AstmProtocol):

    def send_trames(self, trames):
        """Envoie une liste de trame (message) sur la connexion.
        Les trames sont construites avec `build_trames` puis envoyés
        suivant le protocole automate::

            -> <enq>
            <- <ack>
            -> DATA
            <- <ack>
            -> <eot>

        :return: False si erreur sinon True (TODO: exceptions)
        :param trames: liste de trames à envoyer
        """

        self.write(ENQ)
        data = self.read(timeout=15)

        if not data:
            self.write(EOT)
            return False

        if data[0] == ENQ:
            # L'automate a la priorité, le driver attends un temps
            # aléatoire entre 10 et 20 secondes
            self.rtimeout = random.randint(10, 20)
            self.logger.info("ENQ clash: wait %ds before re-sending <enq>", self.rtimeout)
            return True

        if data[0] != ACK:
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<ack> expected in %s: wait %ds before re-sending <enq>",
                byte2log(data), self.rtimeout,
            )
            return True

        index = 0

        while True:
            try:
                trame = trames[index]
            except IndexError:
                break

            self.write(trame)
            data = self.read(timeout=15)

            if not data:
                self.write(EOT)
                return False

            if data[0] == NACK:
                max_tries = 3
                essai = 0
                while essai <= max_tries:
                    self.write(trame)
                    data = self.read(timeout=15)
                    self.rtimeout = random.randint(10, 20)
                    if data[0] == ACK:
                        break
                    essai += 1
                if essai == max_tries:
                    # Le DM ne répondra pas
                    self.write(EOT)
                    return True
            elif data[0] not in (ACK, EOT):
                self.rtimeout = random.randint(10, 20)
                self.logger.info(
                    "<ack> or <eot> expected in %s: wait %ds before re-sending <enq>",
                    byte2log(data), self.rtimeout,
                )
                self.write(EOT)
                # Certains automates ne gèrent pas correctement le Nack.
                # Il faut passer à autre chose, l'automate ne répondra pas.
                return True

            index += 1

        self.write(EOT)

        return True
