# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.utils import SOH, ACK, NACK, STX, ETX, byte2log
from driverng.protocol.base import BaseProtocol


class VtSyslabProtocol(BaseProtocol):

    def send_trames(self, trame):
        self.write(trame)
        data = self.read(timeout=10)
        if not data or data[0] not in (ACK, NACK):
            self.logger.info(
                "Unexpected data '%s'",
                byte2log(data),
            )
            return False

        if data[0] == ACK:
            return True

        return False

    def recv_trames(self):

        data = self.read()

        # Gestion du caractère SOH réceptionné après une longue inactivité
        if data and data[0] == SOH:
            self.write(SOH)
        elif not data or data[0] != STX:
            self.logger.warning("<stx> expexted in %s", byte2log(data))
            return False

        if data and data[0] == STX:
            data += self.readline(break_func=lambda s: s and s[-1] == ETX)
            if self.check_trame(data):
                self.write(ACK)
                if data[1] == 'R':
                    return self.get_payload(data)
            else:
                self.write(NACK)
                return False

    def check_trame(self, trame):
        try:
            checksum = ord(trame[-2])
            checksum_expected = self.checksum_xor(trame[1:-2])
        except ValueError:
            self.logger.warning(
                "Missing checksum for %s",
                byte2log(trame),
            )
            return False

        if checksum != checksum_expected:
            self.logger.warning(
                "Bad checksum for %s: (%02X != %02X)",
                byte2log(trame),
                checksum,
                checksum_expected,
            )
            return False

        return True

    def get_payload(self, data):
        # <stx><payload><checksum><etx>
        return data[1:-2]

    def build_trames(self, message):
        # <stx><payload><checksum><etx>
        return b"%c%s%c%c" % (
            STX,
            message,
            self.checksum_xor(message),
            ETX,
        )

    @staticmethod
    def checksum_xor(data):
        # Exclusive XOR of all characters of the message
        # excluded <stx> and <etx> and <checksum>. If the checksum
        # calculated is 0x03 (ETX), or 0x02 (STX) or 0x00 (NULL), the
        # value will be automatically replaced with 0x7F hex value
        chk = 0
        for c in data:
            chk ^= ord(c)
        if chk in (0x02, 0x03, 0x00):
            chk = 0x7F
        return chk

    def checksum(self, data):
        return self.checksum_xor(data)
