# -*- coding: utf8 -*-
from __future__ import unicode_literals

import random

from driverng.utils import byte2log, ENQ, EOT, ACK, STX, ETX
from driverng.protocol.astm import AstmProtocol


class WalkawayProtocol(AstmProtocol):

    def build_trames(self, message):
        """Enveloppe le message avec les caractères de communication
            <stx><payload><cr><lf><checksum><etx>
        """
        trames = []
        payloads = message.splitlines()

        for payload in payloads:

            while payload:
                trame = payload
                payload = b""

                trame = b"%c%s\r\n" % (STX, trame,)
                checksum = self.checksum(trame)
                trame = b"%s%02X%c" % (trame, checksum, ETX,)
                trames.append(trame)

        return trames

    def recv_trames(self):
        """Reception d'une liste de trame ainsi::
            <- <enq>
            -> <ack>
            <- DATA
            -> <ack>
            <- <eot>
        """
        data = self.read(timeout=1)
        if not data or data[0] != ENQ:
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<enq> expected in %s: wait %ds",
                byte2log(data),
                self.rtimeout,
            )
            return False

        self.write(ACK)

        trames = b""

        while True:
            data = ""
            ctrl = self.read(timeout=15)
            if not ctrl:
                return False

            if ctrl[0] == EOT:
                break

            if ctrl[0] != STX:
                self.logger.warning("<stx> expexted in %s", byte2log(ctrl))
                return False

            data = self.readline(break_func=lambda s: s and s[-1] != "\x82" and s[-1] == ETX)
            trames += self.get_payload(data)

            self.write(ACK)
        messages = b"\n".join(trames.splitlines())
        return self.get_messages(messages.splitlines())

    def get_payload(self, trame):
        payload = trame[0:-3]
        payload += b"\r"
        return payload
