# -*- coding: utf8 -*-
from __future__ import unicode_literals

import os
import sys
import time
from driverng.transport.base import (
    BaseFileTransport,
    BaseFileBackend,
)

if not getattr(sys, "frozen", False):
    import smbc
else:
    smbc = None


class Samba(BaseFileBackend):
    def __init__(self, hostname, domain="WORKGROUP", username=None, password=None, logger=None):
        self.hostname = hostname
        self.username = username
        self.password = password
        self.domain = domain
        self.logger = logger
        self._ttl = time.time()

        if smbc is None:
            raise RuntimeError("On ne peut pas utiliser le transport samba sur un agent binaire")

        def auth(*args):
            return (self.domain, self.username, self.password)

        self.ctx = smbc.Context(auth_fn=auth)

        self.logger.info(
            "Connected to smb://%s@%s using domain %s",
            self.username, self.hostname, self.domain)

        super(Samba, self).__init__()

    def _get_smb_url(self, path):
        if path[0] == "/":
            path = path[1:]
        return "smb://{}/{}".format(self.hostname, path)

    def open(self, path, *args, **kwargs):
        return self.ctx.open(self._get_smb_url(path), *args, **kwargs)

    def list(self, path):
        files = []
        for entry in self.ctx.opendir(self._get_smb_url(path)).getdents():
            if entry.smbc_type == smbc.FILE:
                files.append(entry.name)
        return files

    def put(self, data, path):
        remote_file = self.open(path, os.O_CREAT | os.O_WRONLY)
        remote_file.write(data)
        remote_file.close()

    def get(self, path):
        remote_file = self.open(path)
        data = remote_file.read()
        remote_file.close()
        return data

    def delete(self, path):
        self.ctx.unlink(self._get_smb_url(path))


class SambaTransport(BaseFileTransport):
    def __init__(
        self, domain="WORKGROUP", hostname=None, username=None,
        password=None, **kwargs
    ):
        backend = Samba(hostname, domain, username, password, logger=kwargs.get("logger"))
        super(SambaTransport, self).__init__(backend, **kwargs)
